@@ -3,27 +3,27 @@ import platform
 import sys
 import types
 
-if platform.system() == 'Windows': # pragma: no cover
+if platform.system() == 'Windows':  # pragma: no cover
     WIN = True
-else: # pragma: no cover
+else:  # pragma: no cover
     WIN = False
 
-try: # pragma: no cover
+try:  # pragma: no cover
     import __pypy__
     PYPY = True
-except: # pragma: no cover
+except:  # pragma: no cover
     __pypy__ = None
     PYPY = False
 
 try:
     import cPickle as pickle
-except ImportError: # pragma: no cover
+except ImportError:  # pragma: no cover
     import pickle
 
 # True if we are running on Python 3.
 PY3 = sys.version_info[0] == 3
 
-if PY3: # pragma: no cover
+if PY3:
     string_types = str,
     integer_types = int,
     class_types = type,
@@ -43,16 +43,16 @@ def text_(s, encoding='latin-1', errors='strict'):
     ``s.decode(encoding, errors)``, otherwise return ``s``"""
     if isinstance(s, binary_type):
         return s.decode(encoding, errors)
-    return s # pragma: no cover
+    return s
 
 def bytes_(s, encoding='latin-1', errors='strict'):
     """ If ``s`` is an instance of ``text_type``, return
     ``s.encode(encoding, errors)``, otherwise return ``s``"""
-    if isinstance(s, text_type): # pragma: no cover
+    if isinstance(s, text_type):
         return s.encode(encoding, errors)
     return s
 
-if PY3: # pragma: no cover
+if PY3:
     def ascii_native_(s):
         if isinstance(s, text_type):
             s = s.encode('ascii')
@@ -72,7 +72,7 @@ Python 2: If ``s`` is an instance of ``text_type``, return
 """
 
 
-if PY3: # pragma: no cover
+if PY3:
     def native_(s, encoding='latin-1', errors='strict'):
         """ If ``s`` is an instance of ``text_type``, return
         ``s``, otherwise return ``str(s, encoding, errors)``"""
@@ -95,7 +95,7 @@ Python 2: If ``s`` is an instance of ``text_type``, return
 ``s.encode(encoding, errors)``, otherwise return ``str(s)``
 """
 
-if PY3: # pragma: no cover
+if PY3:
     from urllib import parse
     urlparse = parse
     from urllib.parse import quote as url_quote
@@ -112,18 +112,19 @@ else:
     from urllib import unquote as url_unquote
     from urllib import urlencode as url_encode
     from urllib2 import urlopen as url_open
+
     def url_unquote_text(v, encoding='utf-8', errors='replace'): # pragma: no cover
         v = url_unquote(v)
         return v.decode(encoding, errors)
+
     def url_unquote_native(v, encoding='utf-8', errors='replace'): # pragma: no cover
         return native_(url_unquote_text(v, encoding, errors))
-        
 
-if PY3: # pragma: no cover
+
+if PY3:  # pragma: no cover
     import builtins
     exec_ = getattr(builtins, "exec")
 
-
     def reraise(tp, value, tb=None):
         if value is None:
             value = tp
@@ -131,10 +132,9 @@ if PY3: # pragma: no cover
             raise value.with_traceback(tb)
         raise value
 
-
     del builtins
 
-else: # pragma: no cover
+else:  # pragma: no cover
     def exec_(code, globs=None, locs=None):
         """Execute code in a namespace."""
         if globs is None:
@@ -147,35 +147,38 @@ else: # pragma: no cover
             locs = globs
         exec("""exec code in globs, locs""")
 
-
     exec_("""def reraise(tp, value, tb=None):
     raise tp, value, tb
 """)
 
 
-if PY3: # pragma: no cover
+if PY3:  # pragma: no cover
     def iteritems_(d):
         return d.items()
+
     def itervalues_(d):
         return d.values()
+
     def iterkeys_(d):
         return d.keys()
-else: # pragma: no cover
+else:  # pragma: no cover
     def iteritems_(d):
         return d.iteritems()
+
     def itervalues_(d):
         return d.itervalues()
+
     def iterkeys_(d):
         return d.iterkeys()
 
 
-if PY3: # pragma: no cover
+if PY3:
     def map_(*arg):
         return list(map(*arg))
 else:
     map_ = map
-    
-if PY3: # pragma: no cover
+
+if PY3:
     def is_nonstr_iter(v):
         if isinstance(v, str):
             return False
@@ -183,46 +186,49 @@ if PY3: # pragma: no cover
 else:
     def is_nonstr_iter(v):
         return hasattr(v, '__iter__')
-    
-if PY3: # pragma: no cover
+
+if PY3:
     im_func = '__func__'
     im_self = '__self__'
 else:
     im_func = 'im_func'
     im_self = 'im_self'
 
-try: # pragma: no cover
+try:
     import configparser
-except ImportError: # pragma: no cover
+except ImportError:
     import ConfigParser as configparser
 
 try:
-    from Cookie import SimpleCookie
-except ImportError: # pragma: no cover
     from http.cookies import SimpleCookie
+except ImportError:
+    from Cookie import SimpleCookie
 
-if PY3: # pragma: no cover
+if PY3:
     from html import escape
 else:
     from cgi import escape
 
-try: # pragma: no cover
-    input_ = raw_input
-except NameError: # pragma: no cover
+if PY3:
     input_ = input
+else:
+    input_ = raw_input
 
+if PY3:
+    from inspect import getfullargspec as getargspec
+else:
+    from inspect import getargspec
 
-try: 
-    from StringIO import StringIO as NativeIO
-except ImportError: # pragma: no cover
+if PY3:
     from io import StringIO as NativeIO
+else:
+    from io import BytesIO as NativeIO
 
 # "json" is not an API; it's here to support older pyramid_debugtoolbar
 # versions which attempt to import it
 import json
 
-    
-if PY3: # pragma: no cover
+if PY3:
     # see PEP 3333 for why we encode WSGI PATH_INFO to latin-1 before
     # decoding it to utf-8
     def decode_path_info(path):
@@ -231,16 +237,47 @@ else:
     def decode_path_info(path):
         return path.decode('utf-8')
 
-if PY3: # pragma: no cover
+if PY3:
     # see PEP 3333 for why we decode the path to latin-1 
     from urllib.parse import unquote_to_bytes
+
     def unquote_bytes_to_wsgi(bytestring):
         return unquote_to_bytes(bytestring).decode('latin-1')
 else:
     from urlparse import unquote as unquote_to_bytes
+
     def unquote_bytes_to_wsgi(bytestring):
         return unquote_to_bytes(bytestring)
 
+
 def is_bound_method(ob):
     return inspect.ismethod(ob) and getattr(ob, im_self, None) is not None
 
+# support annotations and keyword-only arguments in PY3
+if PY3: # pragma: no cover
+    from inspect import getfullargspec as getargspec
+else:
+    from inspect import getargspec
+
+if PY3: # pragma: no cover
+    from itertools import zip_longest
+else:
+    from itertools import izip_longest as zip_longest
+
+def is_unbound_method(fn):
+    """
+    This consistently verifies that the callable is bound to a
+    class.
+    """
+    is_bound = is_bound_method(fn)
+
+    if not is_bound and inspect.isroutine(fn):
+        spec = getargspec(fn)
+        has_self = len(spec.args) > 0 and spec.args[0] == 'self'
+
+        if PY3 and inspect.isfunction(fn) and has_self:  # pragma: no cover
+            return True
+        elif inspect.ismethod(fn):
+            return True
+
+    return False
