@@ -1,3 +1,6 @@
+import functools
+
+
 class reify(object):
     """ Use as a class method decorator.  It operates almost exactly like the
     Python ``@property`` decorator, but it puts the result of the method it
@@ -10,28 +13,23 @@ class reify(object):
        class Foo(object):
            @reify
            def jammy(self):
-               print 'jammy called'
+               print('jammy called')
                return 1
 
     And usage of Foo:
 
-    .. code-block:: text
-
-       >>> f = Foo()
-       >>> v = f.jammy
-       'jammy called'
-       >>> print v
-       1
-       >>> f.jammy
-       1
-       >>> # jammy func not called the second time; it replaced itself with 1
+    >>> f = Foo()
+    >>> v = f.jammy
+    'jammy called'
+    >>> print(v)
+    1
+    >>> f.jammy
+    1
+    >>> # jammy func not called the second time; it replaced itself with 1
     """
     def __init__(self, wrapped):
         self.wrapped = wrapped
-        try:
-            self.__doc__ = wrapped.__doc__
-        except: # pragma: no cover
-            pass
+        functools.update_wrapper(self, wrapped)
 
     def __get__(self, inst, objtype=None):
         if inst is None:
