@@ -3,11 +3,16 @@ from pyramid.compat import (
     binary_type,
     is_nonstr_iter,
     url_quote as _url_quote,
-    url_quote_plus as quote_plus, # bw compat api (dnr)
+    url_quote_plus as _quote_plus,
     )
 
-def url_quote(s, safe=''): # bw compat api
-    return _url_quote(s, safe=safe)
+def url_quote(val, safe=''): # bw compat api
+    cls = val.__class__
+    if cls is text_type:
+        val = val.encode('utf-8')
+    elif cls is not binary_type:
+        val = str(val).encode('utf-8')
+    return _url_quote(val, safe=safe)
 
 def urlencode(query, doseq=True):
     """
@@ -32,6 +37,10 @@ def urlencode(query, doseq=True):
 
     See the Python stdlib documentation for ``urllib.urlencode`` for
     more information.
+
+    .. versionchanged:: 1.5
+       In a key/value pair, if the value is ``None`` then it will be
+       dropped from the resulting output.
     """
     try:
         # presumed to be a dictionary
@@ -43,26 +52,28 @@ def urlencode(query, doseq=True):
     prefix = ''
 
     for (k, v) in query:
-        k = _enc(k)
+        k = quote_plus(k)
 
         if is_nonstr_iter(v):
             for x in v:
-                x = _enc(x)
+                x = quote_plus(x)
                 result += '%s%s=%s' % (prefix, k, x)
                 prefix = '&'
+        elif v is None:
+            result += '%s%s=' % (prefix, k)
         else:
-            v = _enc(v)
+            v = quote_plus(v)
             result += '%s%s=%s' % (prefix, k, v)
 
         prefix = '&'
 
     return result
 
-def _enc(val):
+# bw compat api (dnr)
+def quote_plus(val, safe=''):
     cls = val.__class__
     if cls is text_type:
         val = val.encode('utf-8')
     elif cls is not binary_type:
         val = str(val).encode('utf-8')
-    return quote_plus(val)
-
+    return _quote_plus(val, safe=safe)
