@@ -52,6 +52,9 @@ Exception
         * 422 - HTTPUnprocessableEntity
         * 423 - HTTPLocked
         * 424 - HTTPFailedDependency
+        * 428 - HTTPPreconditionRequired
+        * 429 - HTTPTooManyRequests
+        * 431 - HTTPRequestHeaderFieldsTooLarge
       HTTPServerError
         * 500 - HTTPInternalServerError
         * 501 - HTTPNotImplemented
@@ -157,6 +160,13 @@ class HTTPException(Response, Exception):
     # title = 'OK'
     # explanation = 'why this happens'
     # body_template_obj = Template('response template')
+    #
+    # This class itself uses the error code "520" with the error message/title
+    # of "Unknown Error". This is not an RFC standard, however it is
+    # implemented in practice. Sub-classes should be overriding the default
+    # values and 520 should not be seen in the wild from Pyramid applications.
+    # Due to changes in WebOb, a code of "None" is not valid, and WebOb due to
+    # more strict error checking rejects it now.
 
     # differences from webob.exc.WSGIHTTPException:
     #
@@ -175,8 +185,8 @@ class HTTPException(Response, Exception):
     #
     # - documentation improvements (Pyramid-specific docstrings where necessary)
     #
-    code = None
-    title = None
+    code = 520
+    title = 'Unknown Error'
     explanation = ''
     body_template_obj = Template('''\
 ${explanation}${br}${br}
@@ -559,10 +569,6 @@ class HTTPClientError(HTTPError):
     a bug.  A server-side traceback is not warranted.  Unless specialized,
     this is a '400 Bad Request'
     """
-    code = 400
-    title = 'Bad Request'
-    explanation = ('The server could not comply with the request since '
-                   'it is either malformed or otherwise incorrect.')
 
 class HTTPBadRequest(HTTPClientError):
     """
@@ -573,7 +579,10 @@ class HTTPBadRequest(HTTPClientError):
 
     code: 400, title: Bad Request
     """
-    pass
+    code = 400
+    title = 'Bad Request'
+    explanation = ('The server could not comply with the request since '
+                   'it is either malformed or otherwise incorrect.')
 
 class HTTPUnauthorized(HTTPClientError):
     """
@@ -868,7 +877,12 @@ class HTTPUnprocessableEntity(HTTPClientError):
     subclass of :class:`~HTTPClientError`
 
     This indicates that the server is unable to process the contained
-    instructions. Only for WebDAV.
+    instructions. 
+
+    May be used to notify the client that their JSON/XML is well formed, but
+    not correct for the current request.
+
+    See RFC4918 section 11 for more information.
     
     code: 422, title: Unprocessable Entity
     """
@@ -881,7 +895,7 @@ class HTTPLocked(HTTPClientError):
     """
     subclass of :class:`~HTTPClientError`
 
-    This indicates that the resource is locked. Only for WebDAV
+    This indicates that the resource is locked.
     
     code: 423, title: Locked
     """
@@ -896,7 +910,6 @@ class HTTPFailedDependency(HTTPClientError):
 
     This indicates that the method could not be performed because the
     requested action depended on another action and that action failed.
-    Only for WebDAV.
     
     code: 424, title: Failed Dependency
     """
@@ -907,6 +920,62 @@ class HTTPFailedDependency(HTTPClientError):
         'The method could not be performed because the requested '
         'action dependended on another action and that action failed')
 
+class HTTPPreconditionRequired(HTTPClientError):
+    """
+    subclass of :class:`~HTTPClientError`
+
+    This indicates that the origin server requires the
+    request to be conditional.
+
+    Its typical use is to avoid the "lost update" problem, where a client
+    GETs a resource's state, modifies it, and PUTs it back to the server,
+    when meanwhile a third party has modified the state on the server,
+    leading to a conflict.  By requiring requests to be conditional, the
+    server can assure that clients are working with the correct copies.
+
+    RFC 6585.3
+
+    code: 428, title: Precondition Required
+    """
+    code = 428
+    title = 'Precondition Required'
+    explanation = (
+        'The origin server requires the request to be conditional.')
+
+class HTTPTooManyRequests(HTTPClientError):
+    """
+    subclass of :class:`~HTTPClientError`
+
+    This indicates that the user has sent too many
+    requests in a given amount of time ("rate limiting").
+
+    RFC 6585.4
+
+    code: 429, title: Too Many Requests
+    """
+    code = 429
+    title = 'Too Many Requests'
+    explanation = (
+        'The action could not be performed because there were too '
+        'many requests by the client.')
+
+class HTTPRequestHeaderFieldsTooLarge(HTTPClientError):
+    """
+    subclass of :class:`~HTTPClientError`
+
+    This indicates that the server is unwilling to process
+    the request because its header fields are too large.  The request MAY
+    be resubmitted after reducing the size of the request header fields.
+
+    RFC 6585.5
+
+    code: 431, title: Request Header Fields Too Large
+    """
+    code = 431
+    title = 'Request Header Fields Too Large'
+    explanation = (
+        'The requests header fields were too large.')
+
 ############################################################
 ## 5xx Server Error
 ############################################################
@@ -925,15 +994,14 @@ class HTTPServerError(HTTPError):
     This is an error condition in which the server is presumed to be
     in-error.  Unless specialized, this is a '500 Internal Server Error'.
     """
+
+class HTTPInternalServerError(HTTPServerError):
     code = 500
     title = 'Internal Server Error'
     explanation = (
       'The server has either erred or is incapable of performing '
       'the requested operation.')
 
-class HTTPInternalServerError(HTTPServerError):
-    pass
-
 class HTTPNotImplemented(HTTPServerError):
     """
     subclass of :class:`~HTTPServerError`
