@@ -33,8 +33,12 @@ def package_name(pkg_or_module):
     name of the package itself."""
     if pkg_or_module is None or pkg_or_module.__name__ == '__main__':
         return '__main__'
-    pkg_filename = pkg_or_module.__file__
     pkg_name = pkg_or_module.__name__
+    pkg_filename = getattr(pkg_or_module, '__file__', None)
+    if pkg_filename is None:
+        # Namespace packages do not have __init__.py* files,
+        # and so have no __file__ attribute
+        return pkg_name
     splitted = os.path.split(pkg_filename)
     if splitted[-1] in init_names:
         # it's a package
@@ -114,7 +118,7 @@ class AssetResolver(Resolver):
     """ A class used to resolve an :term:`asset specification` to an
     :term:`asset descriptor`.
 
-    .. note:: This API is new as of Pyramid 1.3.
+    .. versionadded:: 1.3
 
     The constructor accepts a single argument named ``package`` which may be
     any of:
@@ -133,20 +137,19 @@ class AssetResolver(Resolver):
     to the :meth:`~pyramid.path.AssetResolver.resolve` method.  An asset
     specification without a colon in it is treated as relative.
 
-    If the value ``None`` is supplied as the ``package``, the resolver will
+    If ``package`` is ``None``, the resolver will
     only be able to resolve fully qualified (not relative) asset
     specifications.  Any attempt to resolve a relative asset specification
-    when the ``package`` is ``None`` will result in an :exc:`ValueError`
-    exception.
+    will result in an :exc:`ValueError` exception.
 
-    If the value :attr:`pyramid.path.CALLER_PACKAGE` is supplied as the
-    ``package``, the resolver will treat relative asset specifications as
+    If ``package`` is :attr:`pyramid.path.CALLER_PACKAGE`,
+    the resolver will treat relative asset specifications as
     relative to the caller of the :meth:`~pyramid.path.AssetResolver.resolve`
     method.
 
-    If a *module* or *module name* (as opposed to a package or package name)
-    is supplied as ``package``, its containing package is computed and this
-    package used to derive the package name (all names are resolved relative
+    If ``package`` is a *module* or *module name* (as opposed to a package or
+    package name), its containing package is computed and this
+    package is used to derive the package name (all names are resolved relative
     to packages, never to modules).  For example, if the ``package`` argument
     to this type was passed the string ``xml.dom.expatbuilder``, and
     ``template.pt`` is supplied to the
@@ -154,8 +157,8 @@ class AssetResolver(Resolver):
     asset spec would be ``xml.minidom:template.pt``, because
     ``xml.dom.expatbuilder`` is a module object, not a package object.
 
-    If a *package* or *package name* (as opposed to a module or module name)
-    is supplied as ``package``, this package will be used to compute relative
+    If ``package`` is a *package* or *package name* (as opposed to a module or
+    module name), this package will be used to compute relative
     asset specifications.  For example, if the ``package`` argument to this
     type was passed the string ``xml.dom``, and ``template.pt`` is supplied
     to the :meth:`~pyramid.path.AssetResolver.resolve` method, the resulting
@@ -182,7 +185,7 @@ class AssetResolver(Resolver):
 
            a = AssetResolver('myproject')
            resolver = a.resolve('templates/foo.pt')
-           print resolver.abspath()
+           print(resolver.abspath())
            # -> /path/to/myproject/templates/foo.pt
 
         If the AssetResolver is constructed without a ``package`` argument of
@@ -209,7 +212,7 @@ class DottedNameResolver(Resolver):
     """ A class used to resolve a :term:`dotted Python name` to a package or
     module object.
 
-    .. note:: This API is new as of Pyramid 1.3.
+    .. versionadded:: 1.3
 
     The constructor accepts a single argument named ``package`` which may be
     any of:
@@ -229,18 +232,17 @@ class DottedNameResolver(Resolver):
     which has a ``.`` (dot) or ``:`` (colon) as its first character is
     treated as relative.
 
-    If the value ``None`` is supplied as the ``package``, the resolver will
-    only be able to resolve fully qualified (not relative) names.  Any
-    attempt to resolve a relative name when the ``package`` is ``None`` will
-    result in an :exc:`ValueError` exception.
+    If ``package`` is ``None``, the resolver will only be able to resolve
+    fully qualified (not relative) names.  Any attempt to resolve a
+    relative name will result in an :exc:`ValueError` exception.
 
-    If the value :attr:`pyramid.path.CALLER_PACKAGE` is supplied as the
-    ``package``, the resolver will treat relative dotted names as relative to
+    If ``package`` is :attr:`pyramid.path.CALLER_PACKAGE`,
+    the resolver will treat relative dotted names as relative to
     the caller of the :meth:`~pyramid.path.DottedNameResolver.resolve`
     method.
 
-    If a *module* or *module name* (as opposed to a package or package name)
-    is supplied as ``package``, its containing package is computed and this
+    If ``package`` is a *module* or *module name* (as opposed to a package or
+    package name), its containing package is computed and this
     package used to derive the package name (all names are resolved relative
     to packages, never to modules).  For example, if the ``package`` argument
     to this type was passed the string ``xml.dom.expatbuilder``, and
@@ -249,8 +251,8 @@ class DottedNameResolver(Resolver):
     import would be for ``xml.minidom``, because ``xml.dom.expatbuilder`` is
     a module object, not a package object.
 
-    If a *package* or *package name* (as opposed to a module or module name)
-    is supplied as ``package``, this package will be used to relative compute
+    If ``package`` is a *package* or *package name* (as opposed to a module or
+    module name), this package will be used to relative compute
     dotted names.  For example, if the ``package`` argument to this type was
     passed the string ``xml.dom``, and ``.minidom`` is supplied to the
     :meth:`~pyramid.path.DottedNameResolver.resolve` method, the resulting
@@ -326,7 +328,7 @@ class DottedNameResolver(Resolver):
 
     def _pkg_resources_style(self, value, package):
         """ package.module:attr style """
-        if value.startswith('.') or value.startswith(':'):
+        if value.startswith(('.', ':')):
             if not package:
                 raise ValueError(
                     'relative name %r irresolveable without package' % (value,)
@@ -335,8 +337,14 @@ class DottedNameResolver(Resolver):
                 value = package.__name__
             else:
                 value = package.__name__ + value
-        return pkg_resources.EntryPoint.parse(
-            'x=%s' % value).load(False)
+        # Calling EntryPoint.load with an argument is deprecated.
+        # See https://pythonhosted.org/setuptools/history.html#id8
+        ep = pkg_resources.EntryPoint.parse('x=%s' % value)
+        if hasattr(ep, 'resolve'):
+            # setuptools>=10.2
+            return ep.resolve()  # pragma: NO COVER
+        else:
+            return ep.load(False)  # pragma: NO COVER
 
     def _zope_dottedname_style(self, value, package):
         """ package.module.attr style """
@@ -388,7 +396,8 @@ class PkgResourcesAssetDescriptor(object):
         return '%s:%s' % (self.pkg_name, self.path)
 
     def abspath(self):
-        return self.pkg_resources.resource_filename(self.pkg_name, self.path)
+        return os.path.abspath(
+            self.pkg_resources.resource_filename(self.pkg_name, self.path))
 
     def stream(self):
         return self.pkg_resources.resource_stream(self.pkg_name, self.path)
