@@ -5,7 +5,7 @@ import textwrap
 
 from pyramid.compat import url_unquote
 from pyramid.request import Request
-from pyramid.paster import get_app
+from pyramid.paster import get_app, setup_logging
 from pyramid.scripts.common import parse_vars
 
 def main(argv=sys.argv, quiet=False):
@@ -14,7 +14,7 @@ def main(argv=sys.argv, quiet=False):
 
 class PRequestCommand(object):
     description = """\
-    Run a request for the described application.
+    Submit a HTTP request to a web application.
 
     This command makes an artifical request to a web application that uses a
     PasteDeploy (.ini) configuration file for the server and application.
@@ -97,12 +97,18 @@ class PRequestCommand(object):
         if not self.quiet:
             print(msg)
 
+    def configure_logging(self, app_spec):
+        setup_logging(app_spec)
+
     def run(self):
         if not len(self.args) >= 2:
             self.out('You must provide at least two arguments')
             return 2
         app_spec = self.args[0]
         path = self.args[1]
+
+        self.configure_logging(app_spec)
+
         if not path.startswith('/'):
             path = '/' + path
 
