@@ -125,21 +125,33 @@ def check_csrf_token(request,
 
     .. versionadded:: 1.4a2
     """
-    supplied_token = request.params.get(token, request.headers.get(header))
-    if supplied_token != request.session.get_csrf_token():
+    supplied_token = request.params.get(token, request.headers.get(header, ""))
+    if strings_differ(request.session.get_csrf_token(), supplied_token):
         if raises:
             raise BadCSRFToken('check_csrf_token(): Invalid token')
         return False
     return True
 
 class PickleSerializer(object):
-    """ A Webob cookie serializer that uses the pickle protocol to dump Python
-    data to bytes."""
+    """ A serializer that uses the pickle protocol to dump Python
+    data to bytes.
+
+    This is the default serializer used by Pyramid.
+
+    ``protocol`` may be specified to control the version of pickle used.
+    Defaults to :attr:`pickle.HIGHEST_PROTOCOL`.
+
+    """
+    def __init__(self, protocol=pickle.HIGHEST_PROTOCOL):
+        self.protocol = protocol
+
     def loads(self, bstruct):
+        """Accept bytes and return a Python object."""
         return pickle.loads(bstruct)
 
     def dumps(self, appstruct):
-        return pickle.dumps(appstruct, pickle.HIGHEST_PROTOCOL)
+        """Accept a Python object and return bytes."""
+        return pickle.dumps(appstruct, self.protocol)
 
 def BaseCookieSessionFactory(
     serializer,
@@ -238,14 +250,14 @@ def BaseCookieSessionFactory(
 
         # configuration parameters
         _cookie_name = cookie_name
-        _cookie_max_age = max_age
+        _cookie_max_age = max_age if max_age is None else int(max_age)
         _cookie_path = path
         _cookie_domain = domain
         _cookie_secure = secure
         _cookie_httponly = httponly
         _cookie_on_exception = set_on_exception
-        _timeout = timeout
-        _reissue_time = reissue_time
+        _timeout = timeout if timeout is None else int(timeout)
+        _reissue_time = reissue_time if reissue_time is None else int(reissue_time)
 
         # dirty flag
         _dirty = False
