@@ -1,5 +1,6 @@
 from webob import Response
 from pyramid.httpexceptions import HTTPForbidden
+from pyramid.compat import bytes_
 
 def x_view(request): # pragma: no cover
      return Response('this is private!')
@@ -9,13 +10,13 @@ def forbidden_view(context, request):
      result = context.result
      message = msg + '\n' + str(result)
      resp = HTTPForbidden()
-     resp.body = message
+     resp.body = bytes_(message)
      return resp
 
 def includeme(config):
      from pyramid.authentication import AuthTktAuthenticationPolicy
      from pyramid.authorization import ACLAuthorizationPolicy
-     authn_policy = AuthTktAuthenticationPolicy('seekr1t')
+     authn_policy = AuthTktAuthenticationPolicy('seekr1t', hashalg='sha512')
      authz_policy = ACLAuthorizationPolicy()
      config._set_authentication_policy(authn_policy)
      config._set_authorization_policy(authz_policy)
