@@ -1,6 +1,6 @@
-from cgi import escape
+from pyramid.compat import escape
 from pyramid.security import view_execution_permitted
-from webob import Response
+from pyramid.response import Response
 
 def x_view(request): # pragma: no cover
      return Response('this is private!')
@@ -14,7 +14,7 @@ def test(context, request):
 def includeme(config):
      from pyramid.authentication import AuthTktAuthenticationPolicy
      from pyramid.authorization import ACLAuthorizationPolicy
-     authn_policy = AuthTktAuthenticationPolicy('seekt1t')
+     authn_policy = AuthTktAuthenticationPolicy('seekt1t', hashalg='sha512')
      authz_policy = ACLAuthorizationPolicy()
      config.set_authentication_policy(authn_policy)
      config.set_authorization_policy(authz_policy)
