@@ -1,6 +1,6 @@
 from pyramid.view import view_config
 
-@view_config(name='one', renderer='one.pt')
+@view_config(name='one', renderer='json')
 def one(request):
     return {'name':'One!'}
 
