@@ -1,11 +1,11 @@
 from pyramid.view import view_config
 
-@view_config(renderer='templates/foo.pt', name='first')
+@view_config(renderer='json', name='first')
 def first(request):
     return {'result':'OK1'}
 
 @view_config(
-    renderer='pyramid.tests.pkgs.viewdecoratorapp.views:templates/foo.pt',
+    renderer='json',
              name='second')
 def second(request):
     return {'result':'OK2'}
