@@ -1,6 +1,6 @@
 # package
 
-from zope.interface import implements
+from zope.interface import implementer
 from zope.interface import Interface
 
 class IFactory(Interface):
@@ -23,8 +23,8 @@ includeme = dummy_include
 class DummyContext:
     pass
 
+@implementer(IFactory)
 class DummyFactory(object):
-    implements(IFactory)
     def __call__(self):
         """ """
 
@@ -43,3 +43,11 @@ def dummy_extend(config, discrim):
 def dummy_extend2(config, discrim):
     config.action(discrim, None, config.registry)
 
+from functools import partial
+dummy_partial = partial(dummy_extend, discrim='partial')
+
+class DummyCallable(object):
+    def __call__(self, config, discrim):
+        config.action(discrim, None, config.package)
+dummy_callable = DummyCallable()
+
