@@ -1,7 +1,4 @@
 import unittest
-import warnings
-
-from pyramid.tests.test_config import dummyfactory
 
 class TestRenderingConfiguratorMixin(unittest.TestCase):
     def _makeOne(self, *arg, **kw):
@@ -9,27 +6,15 @@ class TestRenderingConfiguratorMixin(unittest.TestCase):
         config = Configurator(*arg, **kw)
         return config
 
-    def test_set_renderer_globals_factory(self):
-        from pyramid.interfaces import IRendererGlobalsFactory
+    def test_add_default_renderers(self):
+        from pyramid.config.rendering import DEFAULT_RENDERERS
+        from pyramid.interfaces import IRendererFactory
         config = self._makeOne(autocommit=True)
-        factory = object()
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore')
-            config.set_renderer_globals_factory(factory)
-        self.assertEqual(
-            config.registry.getUtility(IRendererGlobalsFactory),
-            factory)
-
-    def test_set_renderer_globals_factory_dottedname(self):
-        from pyramid.interfaces import IRendererGlobalsFactory
-        config = self._makeOne(autocommit=True)
-        with warnings.catch_warnings():
-            warnings.filterwarnings('ignore')
-            config.set_renderer_globals_factory(
-                'pyramid.tests.test_config.dummyfactory')
-        self.assertEqual(
-            config.registry.getUtility(IRendererGlobalsFactory),
-            dummyfactory)
+        config.add_default_renderers()
+        for name, impl in DEFAULT_RENDERERS:
+            self.assertTrue(
+                config.registry.queryUtility(IRendererFactory, name) is not None
+                )
 
     def test_add_renderer(self):
         from pyramid.interfaces import IRendererFactory
