@@ -364,7 +364,24 @@ class TestPredicateList(unittest.TestCase):
     def test_unknown_predicate(self):
         from pyramid.exceptions import ConfigurationError
         self.assertRaises(ConfigurationError, self._callFUT, unknown=1)
-        
+
+    def test_notted(self):
+        from pyramid.config import not_
+        from pyramid.testing import DummyRequest
+        request = DummyRequest()
+        _, predicates, _ = self._callFUT(
+            xhr='xhr',
+            request_method=not_('POST'),
+            header=not_('header'),
+            )
+        self.assertEqual(predicates[0].text(), 'xhr = True')
+        self.assertEqual(predicates[1].text(),
+                         "!request_method = POST")
+        self.assertEqual(predicates[2].text(), '!header header')
+        self.assertEqual(predicates[1](None, request), True)
+        self.assertEqual(predicates[2](None, request), True)
+
+
 class Test_takes_one_arg(unittest.TestCase):
     def _callFUT(self, view, attr=None, argname=None):
         from pyramid.config.util import takes_one_arg
@@ -543,7 +560,7 @@ class Test_takes_one_arg(unittest.TestCase):
         class Foo: pass
         foo = Foo()
         self.assertFalse(self._callFUT(foo))
-    
+
     def test_method_onearg_named_request(self):
         class Foo:
             def method(self, request):
@@ -551,6 +568,43 @@ class Test_takes_one_arg(unittest.TestCase):
         foo = Foo()
         self.assertTrue(self._callFUT(foo.method))
 
+    def test_function_annotations(self):
+        def foo(bar):
+            """ """
+        # avoid SyntaxErrors in python2, this if effectively nop
+        getattr(foo, '__annotations__', {}).update({'bar': 'baz'})
+        self.assertTrue(self._callFUT(foo))
+
+class TestNotted(unittest.TestCase):
+    def _makeOne(self, predicate):
+        from pyramid.config.util import Notted
+        return Notted(predicate)
+
+    def test_it_with_phash_val(self):
+        pred = DummyPredicate('val')
+        inst = self._makeOne(pred)
+        self.assertEqual(inst.text(), '!val')
+        self.assertEqual(inst.phash(), '!val')
+        self.assertEqual(inst(None, None), False)
+
+    def test_it_without_phash_val(self):
+        pred = DummyPredicate('')
+        inst = self._makeOne(pred)
+        self.assertEqual(inst.text(), '')
+        self.assertEqual(inst.phash(), '')
+        self.assertEqual(inst(None, None), True)
+
+class DummyPredicate(object):
+    def __init__(self, result):
+        self.result = result
+
+    def text(self):
+        return self.result
+
+    phash = text
+
+    def __call__(self, context, request):
+        return True
 
 class DummyCustomPredicate(object):
     def __init__(self):
@@ -579,4 +633,4 @@ class DummyRequest:
 class DummyConfigurator(object):
     def maybe_dotted(self, thing):
         return thing
-    
+
