@@ -1,4 +1,8 @@
 import unittest
+from pyramid.compat import (
+    text_,
+    native_,
+    )
 
 class UrlEncodeTests(unittest.TestCase):
     def _callFUT(self, query, doseq=False):
@@ -10,17 +14,17 @@ class UrlEncodeTests(unittest.TestCase):
         self.assertEqual(result, 'a=1&b=2')
 
     def test_unicode_key(self):
-        la = unicode('LaPe\xc3\xb1a', 'utf-8')
+        la = text_(b'LaPe\xc3\xb1a', 'utf-8')
         result = self._callFUT([(la, 1), ('b',2)])
         self.assertEqual(result, 'LaPe%C3%B1a=1&b=2')
 
     def test_unicode_val_single(self):
-        la = unicode('LaPe\xc3\xb1a', 'utf-8')
+        la = text_(b'LaPe\xc3\xb1a', 'utf-8')
         result = self._callFUT([('a', la), ('b',2)])
         self.assertEqual(result, 'a=LaPe%C3%B1a&b=2')
 
     def test_unicode_val_multiple(self):
-        la = [unicode('LaPe\xc3\xb1a', 'utf-8')] * 2
+        la = [text_(b'LaPe\xc3\xb1a', 'utf-8')] * 2
         result = self._callFUT([('a', la), ('b',2)], doseq=True)
         self.assertEqual(result, 'a=LaPe%C3%B1a&a=LaPe%C3%B1a&b=2')
 
@@ -37,34 +41,39 @@ class UrlEncodeTests(unittest.TestCase):
         result = self._callFUT({'a':1})
         self.assertEqual(result, 'a=1')
 
+    def test_None_value(self):
+        result = self._callFUT([('a', None)])
+        self.assertEqual(result, 'a=')
+
+    def test_None_value_with_prefix(self):
+        result = self._callFUT([('a', '1'), ('b', None)])
+        self.assertEqual(result, 'a=1&b=')
+
+    def test_None_value_with_prefix_values(self):
+        result = self._callFUT([('a', '1'), ('b', None), ('c', None)])
+        self.assertEqual(result, 'a=1&b=&c=')
+
 class URLQuoteTests(unittest.TestCase):
     def _callFUT(self, val, safe=''):
         from pyramid.encode import url_quote
         return url_quote(val, safe)
 
-    def test_it_default(self):
-        la = 'La/Pe\xc3\xb1a'
+    def test_it_bytes(self):
+        la = b'La/Pe\xc3\xb1a'
         result = self._callFUT(la)
         self.assertEqual(result, 'La%2FPe%C3%B1a')
         
+    def test_it_native(self):
+        la = native_(b'La/Pe\xc3\xb1a', 'utf-8')
+        result = self._callFUT(la)
+        self.assertEqual(result, 'La%2FPe%C3%B1a')
+
     def test_it_with_safe(self):
-        la = 'La/Pe\xc3\xb1a'
+        la = b'La/Pe\xc3\xb1a'
         result = self._callFUT(la, '/')
         self.assertEqual(result, 'La/Pe%C3%B1a')
 
-class TestQuotePlus(unittest.TestCase):
-    def _callFUT(self, val, safe=''):
-        from pyramid.encode import quote_plus
-        return quote_plus(val, safe)
-    
-    def test_it_default(self):
-        la = 'La Pe\xc3\xb1a'
-        result = self._callFUT(la)
-        self.assertEqual(result, 'La+Pe%C3%B1a')
-        
-    def test_it_with_safe(self):
-        la = 'La /Pe\xc3\xb1a'
+    def test_it_with_nonstr_nonbinary(self):
+        la = None
         result = self._callFUT(la, '/')
-        self.assertEqual(result, 'La+/Pe%C3%B1a')
-
-        
+        self.assertEqual(result, 'None')
