@@ -376,7 +376,7 @@ class TestDottedNameResolver(unittest.TestCase):
 
     def test_zope_dottedname_style_resolve_builtin(self):
         typ = self._makeOne()
-        if PY3: # pragma: no cover
+        if PY3:
             result = typ._zope_dottedname_style('builtins.str', None)
         else:
             result = typ._zope_dottedname_style('__builtin__.str', None)
