@@ -12,11 +12,6 @@ class TestPyramidTemplate(unittest.TestCase):
         self.assertTrue(vars['random_string'])
         self.assertEqual(vars['package_logger'], 'one')
 
-    def test_pre_site(self):
-        inst = self._makeOne()
-        vars = {'package':'site'}
-        self.assertRaises(ValueError, inst.pre, 'command', 'output dir', vars)
-        
     def test_pre_root(self):
         inst = self._makeOne()
         vars = {'package':'root'}
