@@ -1,5 +1,6 @@
 import unittest
 
+
 class TestPCreateCommand(unittest.TestCase):
     def setUp(self):
         from pyramid.compat import NativeIO
@@ -12,10 +13,11 @@ class TestPCreateCommand(unittest.TestCase):
         from pyramid.scripts.pcreate import PCreateCommand
         return PCreateCommand
 
-    def _makeOne(self, *args):
+    def _makeOne(self, *args, **kw):
         effargs = ['pcreate']
         effargs.extend(args)
-        cmd = self._getTargetClass()(effargs)
+        tgt_class = kw.pop('target_class', self._getTargetClass())
+        cmd = tgt_class(effargs, **kw)
         cmd.out = self.out
         return cmd
 
@@ -34,8 +36,13 @@ class TestPCreateCommand(unittest.TestCase):
         out = self.out_.getvalue()
         self.assertTrue(out.startswith('No scaffolds available'))
 
+    def test_run_no_scaffold_no_args(self):
+        cmd = self._makeOne(quiet=True)
+        result = cmd.run()
+        self.assertEqual(result, 2)
+
     def test_run_no_scaffold_name(self):
-        cmd = self._makeOne()
+        cmd = self._makeOne('dummy')
         result = cmd.run()
         self.assertEqual(result, 2)
         out = self.out_.getvalue()
@@ -73,6 +80,23 @@ class TestPCreateCommand(unittest.TestCase):
             {'project': 'Distro', 'egg': 'Distro', 'package': 'distro',
              'pyramid_version': '0.1', 'pyramid_docs_branch':'0.1-branch'})
 
+    def test_scaffold_with_hyphen_in_project_name(self):
+        import os
+        cmd = self._makeOne('-s', 'dummy', 'Distro-')
+        scaffold = DummyScaffold('dummy')
+        cmd.scaffolds = [scaffold]
+        cmd.pyramid_dist = DummyDist("0.1")
+        result = cmd.run()
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            scaffold.output_dir,
+            os.path.normpath(os.path.join(os.getcwd(), 'Distro-'))
+            )
+        self.assertEqual(
+            scaffold.vars,
+            {'project': 'Distro-', 'egg': 'Distro_', 'package': 'distro_',
+             'pyramid_version': '0.1', 'pyramid_docs_branch':'0.1-branch'})
+
     def test_known_scaffold_absolute_path(self):
         import os
         path = os.path.abspath('Distro')
@@ -198,6 +222,48 @@ class TestPCreateCommand(unittest.TestCase):
              'pyramid_version': '0.10.1dev',
              'pyramid_docs_branch': 'master'})
 
+    def test_confirm_override_conflicting_name(self):
+        from pyramid.scripts.pcreate import PCreateCommand
+        class YahInputPCreateCommand(PCreateCommand):
+            def confirm_bad_name(self, pkg_name):
+                return True
+        cmd = self._makeOne('-s', 'dummy', 'Unittest', target_class=YahInputPCreateCommand)
+        scaffold = DummyScaffold('dummy')
+        cmd.scaffolds = [scaffold]
+        cmd.pyramid_dist = DummyDist("0.10.1dev")
+        result = cmd.run()
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            scaffold.vars,
+            {'project': 'Unittest', 'egg': 'Unittest', 'package': 'unittest',
+             'pyramid_version': '0.10.1dev',
+             'pyramid_docs_branch': 'master'})
+
+    def test_force_override_conflicting_name(self):
+        cmd = self._makeOne('-s', 'dummy', 'Unittest', '--ignore-conflicting-name')
+        scaffold = DummyScaffold('dummy')
+        cmd.scaffolds = [scaffold]
+        cmd.pyramid_dist = DummyDist("0.10.1dev")
+        result = cmd.run()
+        self.assertEqual(result, 0)
+        self.assertEqual(
+            scaffold.vars,
+            {'project': 'Unittest', 'egg': 'Unittest', 'package': 'unittest',
+             'pyramid_version': '0.10.1dev',
+             'pyramid_docs_branch': 'master'})
+
+    def test_force_override_site_name(self):
+        from pyramid.scripts.pcreate import PCreateCommand
+        class NayInputPCreateCommand(PCreateCommand):
+            def confirm_bad_name(self, pkg_name):
+                return False
+        cmd = self._makeOne('-s', 'dummy', 'Site', target_class=NayInputPCreateCommand)
+        scaffold = DummyScaffold('dummy')
+        cmd.scaffolds = [scaffold]
+        cmd.pyramid_dist = DummyDist("0.10.1dev")
+        result = cmd.run()
+        self.assertEqual(result, 2)
+
 
 class Test_main(unittest.TestCase):
     def _callFUT(self, argv):
