@@ -1,3 +1,4 @@
+import base64
 import json
 import unittest
 from pyramid import testing
@@ -61,6 +62,18 @@ class SharedCookieSessionTests(object):
         session = self._makeOne(request, timeout=None)
         self.assertEqual(dict(session), {'state': 1})
 
+    def test_timeout_str(self):
+        import time
+        request = testing.DummyRequest()
+        cookieval = self._serialize((time.time() - 5, 0, {'state': 1}))
+        request.cookies['session'] = cookieval
+        session = self._makeOne(request, timeout='1')
+        self.assertEqual(dict(session), {})
+
+    def test_timeout_invalid(self):
+        request = testing.DummyRequest()
+        self.assertRaises(ValueError, self._makeOne, request, timeout='Invalid value')
+
     def test_changed(self):
         request = testing.DummyRequest()
         session = self._makeOne(request)
@@ -277,7 +290,7 @@ class TestBaseCookieSession(SharedCookieSessionTests, unittest.TestCase):
         return BaseCookieSessionFactory(serializer, **kw)(request)
 
     def _serialize(self, value):
-        return json.dumps(value)
+        return base64.b64encode(json.dumps(value).encode('utf-8'))
 
     def test_reissue_not_triggered(self):
         import time
@@ -296,6 +309,23 @@ class TestBaseCookieSession(SharedCookieSessionTests, unittest.TestCase):
         self.assertEqual(session['state'], 1)
         self.assertFalse(session._dirty)
 
+    def test_reissue_str_triggered(self):
+        import time
+        request = testing.DummyRequest()
+        cookieval = self._serialize((time.time() - 2, 0, {'state': 1}))
+        request.cookies['session'] = cookieval
+        session = self._makeOne(request, reissue_time='0')
+        self.assertEqual(session['state'], 1)
+        self.assertTrue(session._dirty)
+
+    def test_reissue_invalid(self):
+        request = testing.DummyRequest()
+        self.assertRaises(ValueError, self._makeOne, request, reissue_time='invalid value')
+
+    def test_cookie_max_age_invalid(self):
+        request = testing.DummyRequest()
+        self.assertRaises(ValueError, self._makeOne, request, max_age='invalid value')
+
 class TestSignedCookieSession(SharedCookieSessionTests, unittest.TestCase):
     def _makeOne(self, request, **kw):
         from pyramid.session import SignedCookieSessionFactory
@@ -330,6 +360,23 @@ class TestSignedCookieSession(SharedCookieSessionTests, unittest.TestCase):
         self.assertEqual(session['state'], 1)
         self.assertFalse(session._dirty)
 
+    def test_reissue_str_triggered(self):
+        import time
+        request = testing.DummyRequest()
+        cookieval = self._serialize((time.time() - 2, 0, {'state': 1}))
+        request.cookies['session'] = cookieval
+        session = self._makeOne(request, reissue_time='0')
+        self.assertEqual(session['state'], 1)
+        self.assertTrue(session._dirty)
+
+    def test_reissue_invalid(self):
+        request = testing.DummyRequest()
+        self.assertRaises(ValueError, self._makeOne, request, reissue_time='invalid value')
+
+    def test_cookie_max_age_invalid(self):
+        request = testing.DummyRequest()
+        self.assertRaises(ValueError, self._makeOne, request, max_age='invalid value')
+
     def test_custom_salt(self):
         import time
         request = testing.DummyRequest()
@@ -521,7 +568,7 @@ class Test_manage_accessed(unittest.TestCase):
         result = wrapper(session, 'a')
         self.assertEqual(result, 1)
         callbacks = request.response_callbacks
-        self.assertEqual(len(callbacks), 0)
+        if callbacks is not None: self.assertEqual(len(callbacks), 0)
 
 class Test_manage_changed(unittest.TestCase):
     def _makeOne(self, wrapped):
@@ -650,10 +697,16 @@ class Test_check_csrf_token(unittest.TestCase):
 
 class DummySerializer(object):
     def dumps(self, value):
-        return json.dumps(value).encode('utf-8')
+        return base64.b64encode(json.dumps(value).encode('utf-8'))
 
     def loads(self, value):
-        return json.loads(value.decode('utf-8'))
+        try:
+            return json.loads(base64.b64decode(value).decode('utf-8'))
+
+        # base64.b64decode raises a TypeError on py2 instead of a ValueError
+        # and a ValueError is required for the session to handle it properly
+        except TypeError:
+            raise ValueError
 
 class DummySessionFactory(dict):
     _dirty = False
