@@ -6,7 +6,6 @@ from pyramid import testing
 
 from pyramid.compat import (
     text_,
-    native_,
     WIN,
     )
 
@@ -46,11 +45,12 @@ class TestURLMethodsMixin(unittest.TestCase):
         from pyramid.interfaces import IResourceURL
         from zope.interface import Interface
         class DummyResourceURL(object):
-            def __init__(self, context, request):
-                self.physical_path = '/context/'
-                self.virtual_path = '/context/'
+            physical_path = '/context/'
+            virtual_path = '/context/'
+            def __init__(self, context, request): pass
         reg.registerAdapter(DummyResourceURL, (Interface, Interface),
                             IResourceURL)
+        return DummyResourceURL
 
     def test_resource_url_root_default(self):
         request = self._makeOne()
@@ -92,6 +92,14 @@ class TestURLMethodsMixin(unittest.TestCase):
         result = request.resource_url(context, 'a b c')
         self.assertEqual(result, 'http://example.com:5432/context/a%20b%20c')
 
+    def test_resource_url_with_query_str(self):
+        request = self._makeOne()
+        self._registerResourceURL(request.registry)
+        context = DummyContext()
+        result = request.resource_url(context, 'a', query='(openlayers)')
+        self.assertEqual(result,
+            'http://example.com:5432/context/a?(openlayers)')
+
     def test_resource_url_with_query_dict(self):
         request = self._makeOne()
         self._registerResourceURL(request.registry)
@@ -148,23 +156,18 @@ class TestURLMethodsMixin(unittest.TestCase):
         request = self._makeOne()
         self._registerResourceURL(request.registry)
         context = DummyContext()
-        uc = text_(b'La Pe\xc3\xb1a', 'utf-8') 
+        uc = text_(b'La Pe\xc3\xb1a', 'utf-8')
         result = request.resource_url(context, anchor=uc)
-        self.assertEqual(
-            result,
-            native_(
-                text_(b'http://example.com:5432/context/#La Pe\xc3\xb1a',
-                      'utf-8'),
-                'utf-8')
-            )
+        self.assertEqual(result,
+                         'http://example.com:5432/context/#La%20Pe%C3%B1a')
 
-    def test_resource_url_anchor_is_not_urlencoded(self):
+    def test_resource_url_anchor_is_urlencoded_safe(self):
         request = self._makeOne()
         self._registerResourceURL(request.registry)
         context = DummyContext()
-        result = request.resource_url(context, anchor=' /#')
+        result = request.resource_url(context, anchor=' /#?&+')
         self.assertEqual(result,
-                         'http://example.com:5432/context/# /#')
+                         'http://example.com:5432/context/#%20/%23?&+')
 
     def test_resource_url_no_IResourceURL_registered(self):
         # falls back to ResourceURL
@@ -255,6 +258,148 @@ class TestURLMethodsMixin(unittest.TestCase):
         root.__resource_url__ = resource_url
         result = request.resource_url(root)
         self.assertEqual(result, 'http://example.com/contextabc/')
+
+    def test_resource_url_with_route_name_no_remainder_on_adapter(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # no virtual_path_tuple on adapter
+        adapter.virtual_path = '/a/b/c/'
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, route_name='foo')
+        self.assertEqual(result, 'http://example.com:5432/1/2/3')
+        self.assertEqual(route.kw, {'traverse': ('', 'a', 'b', 'c', '')})
+
+    def test_resource_url_with_route_name_remainder_on_adapter(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, route_name='foo')
+        self.assertEqual(result, 'http://example.com:5432/1/2/3')
+        self.assertEqual(route.kw, {'traverse': ('', 'a', 'b', 'c', '')})
+
+    def test_resource_url_with_route_name_and_app_url(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, route_name='foo', app_url='app_url')
+        self.assertEqual(result, 'app_url/1/2/3')
+        self.assertEqual(route.kw, {'traverse': ('', 'a', 'b', 'c', '')})
+
+    def test_resource_url_with_route_name_and_scheme_host_port_etc(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, route_name='foo', scheme='scheme',
+                                      host='host', port='port', query={'a':'1'},
+                                      anchor='anchor')
+        self.assertEqual(result, 'scheme://host:port/1/2/3?a=1#anchor')
+        self.assertEqual(route.kw, {'traverse': ('', 'a', 'b', 'c', '')})
+
+    def test_resource_url_with_route_name_and_route_kwargs(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(
+            root, route_name='foo', route_kw={'a':'1', 'b':'2'})
+        self.assertEqual(result, 'http://example.com:5432/1/2/3')
+        self.assertEqual(
+            route.kw,
+            {'traverse': ('', 'a', 'b', 'c', ''),
+             'a':'1',
+             'b':'2'}
+            )
+
+    def test_resource_url_with_route_name_and_elements(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, 'e1', 'e2', route_name='foo')
+        self.assertEqual(result,  'http://example.com:5432/1/2/3/e1/e2')
+        self.assertEqual(route.kw, {'traverse': ('', 'a', 'b', 'c', '')})
+        
+    def test_resource_url_with_route_name_and_remainder_name(self):
+        from pyramid.interfaces import IRoutesMapper
+        environ = {
+            'wsgi.url_scheme':'http',
+            'SERVER_PORT':'8080',
+            'SERVER_NAME':'example.com',
+            }
+        request = self._makeOne(environ)
+        adapter = self._registerResourceURL(request.registry)
+        # virtual_path_tuple on adapter
+        adapter.virtual_path_tuple = ('', 'a', 'b', 'c', '')
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route)
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        root = DummyContext()
+        result = request.resource_url(root, route_name='foo',
+                                      route_remainder_name='fred')
+        self.assertEqual(result, 'http://example.com:5432/1/2/3')
+        self.assertEqual(route.kw, {'fred': ('', 'a', 'b', 'c', '')})
         
     def test_resource_path(self):
         request = self._makeOne()
@@ -305,14 +450,8 @@ class TestURLMethodsMixin(unittest.TestCase):
         request.registry.registerUtility(mapper, IRoutesMapper)
         result = request.route_url('flub', _anchor=b"La Pe\xc3\xb1a")
 
-        self.assertEqual(
-            result,
-            native_(
-                text_(
-                    b'http://example.com:5432/1/2/3#La Pe\xc3\xb1a',
-                    'utf-8'),
-                'utf-8')
-            )
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3#La%20Pe%C3%B1a')
 
     def test_route_url_with_anchor_unicode(self):
         from pyramid.interfaces import IRoutesMapper
@@ -322,14 +461,8 @@ class TestURLMethodsMixin(unittest.TestCase):
         anchor = text_(b'La Pe\xc3\xb1a', 'utf-8')
         result = request.route_url('flub', _anchor=anchor)
 
-        self.assertEqual(
-            result,
-            native_(
-                text_(
-                    b'http://example.com:5432/1/2/3#La Pe\xc3\xb1a',
-                    'utf-8'),
-                'utf-8')
-            )
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3#La%20Pe%C3%B1a')
 
     def test_route_url_with_query(self):
         from pyramid.interfaces import IRoutesMapper
@@ -340,6 +473,15 @@ class TestURLMethodsMixin(unittest.TestCase):
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3?q=1')
 
+    def test_route_url_with_query_str(self):
+        from pyramid.interfaces import IRoutesMapper
+        request = self._makeOne()
+        mapper = DummyRoutesMapper(route=DummyRoute('/1/2/3'))
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.route_url('flub', _query='(openlayers)')
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3?(openlayers)')
+
     def test_route_url_with_empty_query(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -485,6 +627,49 @@ class TestURLMethodsMixin(unittest.TestCase):
         self.assertEqual(result,
                          'http://example.com:5432/1/2/3/extra1/extra2?a=1#foo')
 
+    def test_current_route_url_with_request_query(self):
+        from pyramid.interfaces import IRoutesMapper
+        from webob.multidict import GetDict
+        request = self._makeOne()
+        request.GET = GetDict([('q', '123')], {})
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route=route)
+        request.matched_route = route
+        request.matchdict = {}
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.current_route_url()
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3?q=123')
+
+    def test_current_route_url_with_request_query_duplicate_entries(self):
+        from pyramid.interfaces import IRoutesMapper
+        from webob.multidict import GetDict
+        request = self._makeOne()
+        request.GET = GetDict(
+            [('q', '123'), ('b', '2'), ('b', '2'), ('q', '456')], {})
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route=route)
+        request.matched_route = route
+        request.matchdict = {}
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.current_route_url()
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3?q=123&b=2&b=2&q=456')
+
+    def test_current_route_url_with_query_override(self):
+        from pyramid.interfaces import IRoutesMapper
+        from webob.multidict import GetDict
+        request = self._makeOne()
+        request.GET = GetDict([('q', '123')], {})
+        route = DummyRoute('/1/2/3')
+        mapper = DummyRoutesMapper(route=route)
+        request.matched_route = route
+        request.matchdict = {}
+        request.registry.registerUtility(mapper, IRoutesMapper)
+        result = request.current_route_url(_query={'a':1})
+        self.assertEqual(result,
+                         'http://example.com:5432/1/2/3?a=1')
+
     def test_current_route_path(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -497,7 +682,7 @@ class TestURLMethodsMixin(unittest.TestCase):
         result = request.current_route_path('extra1', 'extra2', _query={'a':1},
                                             _anchor=text_(b"foo"))
         self.assertEqual(result, '/script_name/1/2/3/extra1/extra2?a=1#foo')
-        
+
     def test_route_path_with_elements(self):
         from pyramid.interfaces import IRoutesMapper
         request = self._makeOne()
@@ -569,7 +754,6 @@ class TestURLMethodsMixin(unittest.TestCase):
                          ('pyramid.tests:static/foo.css', request, {}) )
 
     def test_static_url_abspath_integration_with_staticurlinfo(self):
-        import os
         from pyramid.interfaces import IStaticURLInfo
         from pyramid.config.views import StaticURLInfo
         info = StaticURLInfo()
@@ -583,6 +767,20 @@ class TestURLMethodsMixin(unittest.TestCase):
         self.assertEqual(result,
                          'http://example.com:5432/absstatic/test_url.py')
 
+    def test_static_url_noscheme_uses_scheme_from_request(self):
+        from pyramid.interfaces import IStaticURLInfo
+        from pyramid.config.views import StaticURLInfo
+        info = StaticURLInfo()
+        here = os.path.abspath(os.path.dirname(__file__))
+        info.add(self.config, '//subdomain.example.com/static', here)
+        request = self._makeOne({'wsgi.url_scheme': 'https'})
+        registry = request.registry
+        registry.registerUtility(info, IStaticURLInfo)
+        abspath = os.path.join(here, 'test_url.py')
+        result = request.static_url(abspath)
+        self.assertEqual(result,
+                         'https://subdomain.example.com/static/test_url.py')
+
     def test_static_path_abspath(self):
         from pyramid.interfaces import IStaticURLInfo
         request = self._makeOne()
@@ -972,6 +1170,77 @@ class Test_current_route_path(unittest.TestCase):
         self.assertEqual(request.elements, ('abc',))
         self.assertEqual(request.kw, {'_anchor':'abc'})
 
+class Test_external_static_url_integration(unittest.TestCase):
+
+    def setUp(self):
+        self.config = testing.setUp()
+
+    def tearDown(self):
+        testing.tearDown()
+
+    def _makeRequest(self):
+        from pyramid.request import Request
+        return Request.blank('/')
+
+    def test_generate_external_url(self):
+        self.config.add_route('acme', 'https://acme.org/path/{foo}')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertEqual(
+            request.route_url('acme', foo='bar'),
+            'https://acme.org/path/bar')
+
+    def test_generate_external_url_without_scheme(self):
+        self.config.add_route('acme', '//acme.org/path/{foo}')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertEqual(
+            request.route_url('acme', foo='bar'),
+            'http://acme.org/path/bar')
+
+    def test_generate_external_url_with_explicit_scheme(self):
+        self.config.add_route('acme', '//acme.org/path/{foo}')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertEqual(
+            request.route_url('acme', foo='bar', _scheme='https'),
+            'https://acme.org/path/bar')
+
+    def test_generate_external_url_with_explicit_app_url(self):
+        self.config.add_route('acme', 'http://acme.org/path/{foo}')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertRaises(ValueError,
+            request.route_url, 'acme', foo='bar', _app_url='http://fakeme.com')
+
+    def test_generate_external_url_route_path(self):
+        self.config.add_route('acme', 'https://acme.org/path/{foo}')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertRaises(ValueError, request.route_path, 'acme', foo='bar')
+
+    def test_generate_external_url_with_pregenerator(self):
+        def pregenerator(request, elements, kw):
+            kw['_query'] = {'q': 'foo'}
+            return elements, kw
+        self.config.add_route('acme', 'https://acme.org/path/{foo}',
+                              pregenerator=pregenerator)
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertEqual(
+            request.route_url('acme', foo='bar'),
+            'https://acme.org/path/bar?q=foo')
+
+    def test_external_url_with_route_prefix(self):
+        def includeme(config):
+            config.add_route('acme', '//acme.org/{foo}')
+        self.config.include(includeme, route_prefix='some_prefix')
+        request = self._makeRequest()
+        request.registry = self.config.registry
+        self.assertEqual(
+            request.route_url('acme', foo='bar'),
+            'http://acme.org/bar')
+
 class DummyContext(object):
     def __init__(self, next=None):
         self.next = next
