@@ -5,6 +5,8 @@ from zope.interface import implementer
 
 from pyramid import testing
 
+from pyramid.interfaces import IRequest
+
 class BaseTest(object):
     def setUp(self):
         self.config = testing.setUp()
@@ -13,7 +15,6 @@ class BaseTest(object):
         testing.tearDown()
 
     def _registerView(self, reg, app, name):
-        from pyramid.interfaces import IRequest
         from pyramid.interfaces import IViewClassifier
         for_ = (IViewClassifier, IRequest, IContext)
         from pyramid.interfaces import IView
@@ -32,14 +33,11 @@ class BaseTest(object):
         return environ
 
     def _makeRequest(self, **environ):
-        from pyramid.interfaces import IRequest
-        from zope.interface import directlyProvides
-        from webob import Request
+        from pyramid.request import Request
         from pyramid.registry import Registry
         environ = self._makeEnviron(**environ)
         request = Request(environ)
         request.registry = Registry()
-        directlyProvides(request, IRequest)
         return request
 
     def _makeContext(self):
@@ -187,6 +185,20 @@ class RenderViewToResponseTests(BaseTest, unittest.TestCase):
         self.assertEqual(response.status, '200 OK')
         self.assertEqual(response.app_iter, ['anotherview'])
 
+    def test_call_view_with_request_iface_on_request(self):
+        # See https://github.com/Pylons/pyramid/issues/1643
+        from zope.interface import Interface
+        class IWontBeFound(Interface): pass
+        context = self._makeContext()
+        request = self._makeRequest()
+        request.request_iface = IWontBeFound
+        response = DummyResponse('aview')
+        view = make_view(response)
+        self._registerView(request.registry, view, 'aview')
+        response = self._callFUT(context, request, name='aview')
+        self.assertEqual(response.status, '200 OK')
+        self.assertEqual(response.app_iter, ['aview'])
+
 class RenderViewToIterableTests(BaseTest, unittest.TestCase):
     def _callFUT(self, *arg, **kw):
         from pyramid.view import render_view_to_iterable
@@ -676,6 +688,7 @@ def make_view(response):
 
 class DummyRequest:
     exception = None
+    request_iface = IRequest
 
     def __init__(self, environ=None):
         if environ is None:
